@@ -2,7 +2,7 @@
  * @file liferea_shell.c  UI layout handling
  *
  * Copyright (C) 2004-2006 Nathan J. Conrad <t98502@users.sourceforge.net>
- * Copyright (C) 2007-2023 Lars Windolf <lars.windolf@gmx.de>
+ * Copyright (C) 2007-2025 Lars Windolf <lars.windolf@gmx.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +28,6 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
-#include <libpeas/peas-extension-set.h>
 
 #include "browser.h"
 #include "common.h"
@@ -41,23 +40,22 @@
 #include "itemlist.h"
 #include "liferea_application.h"
 #include "net_monitor.h"
-#include "newsbin.h"
-#include "plugins_engine.h"
-#include "render.h"
 #include "social.h"
-#include "vfolder.h"
-#include "fl_sources/node_source.h"
+#include "node_source.h"
+#include "node_providers/newsbin.h"
+#include "node_providers/vfolder.h"
+#include "plugins/plugins_engine.h"
 #include "ui/browser_tabs.h"
 #include "ui/feed_list_view.h"
 #include "ui/icons.h"
 #include "ui/itemview.h"
 #include "ui/item_list_view.h"
 #include "ui/liferea_dialog.h"
-#include "ui/liferea_shell_activatable.h"
 #include "ui/preferences_dialog.h"
 #include "ui/search_dialog.h"
 #include "ui/ui_common.h"
 #include "ui/ui_update.h"
+#include "webkit/liferea_web_view.h"
 
 extern gboolean searchFolderRebuild; /* db.c */
 
@@ -66,6 +64,8 @@ struct _LifereaShell {
 
 	GtkBuilder	*xml;
 
+	LifereaPluginsEngine *plugins;
+
 	GtkWindow	*window;			/*<< Liferea main window */
 	GtkWidget	*toolbar;
 	GtkTreeView	*feedlistViewWidget;
@@ -88,8 +88,6 @@ struct _LifereaShell {
 	ItemView	*itemview;
 	BrowserTabs	*tabs;
 
-	PeasExtensionSet *extensions;		/*<< Plugin management */
-
 	gboolean	fullscreen;				/*<< track fullscreen */
 };
 
@@ -111,6 +109,7 @@ liferea_shell_finalize (GObject *object)
 {
 	LifereaShell *ls = LIFEREA_SHELL (object);
 
+	g_object_unref (ls->plugins);
 	g_object_unref (ls->xml);
 }
 
@@ -236,7 +235,7 @@ liferea_shell_restore_position (void)
 	conf_get_int_value (LAST_WINDOW_WIDTH, &w);
 	conf_get_int_value (LAST_WINDOW_HEIGHT, &h);
 
-	debug4 (DEBUG_GUI, "Retrieved saved setting: size %dx%d position %d:%d", w, h, x, y);
+	debug (DEBUG_GUI, "Retrieved saved setting: size %dx%d position %d:%d", w, h, x, y);
 
 	/* Restore position only if the width and height were saved */
 	if (w != 0 && h != 0) {
@@ -254,12 +253,9 @@ liferea_shell_restore_position (void)
 		else if (y + w < 0)
 			y = 100;
 
-		debug4 (DEBUG_GUI, "Restoring to size %dx%d position %d:%d", w, h, x, y);
+		debug (DEBUG_GUI, "Restoring to size %dx%d position %d:%d", w, h, x, y);
 
 		gtk_window_move (GTK_WINDOW (shell->window), x, y);
-
-		/* load window size */
-		gtk_window_resize (GTK_WINDOW (shell->window), w, h);
 	}
 
 	conf_get_bool_value (LAST_WINDOW_MAXIMIZED, &last_window_maximized);
@@ -281,6 +277,8 @@ liferea_shell_save_position (void)
 	GdkMonitor 		*monitor;
 	GdkRectangle		work_area;
 
+	g_assert(shell);
+
 	/* save pane proportions */
 	pane = liferea_shell_lookup ("leftpane");
 	if (pane) {
@@ -321,7 +319,7 @@ liferea_shell_save_position (void)
 	    y > work_area.height)
 		return;
 
-	debug4 (DEBUG_GUI, "Saving window size and position: %dx%d %d:%d", w, h, x, y);
+	debug (DEBUG_GUI, "Saving window size and position: %dx%d %d:%d", w, h, x, y);
 
 	/* save window position */
 	conf_set_int_value (LAST_WINDOW_X, x);
@@ -427,7 +425,7 @@ liferea_shell_update_node_actions (gpointer obj, gchar *unusedNodeId, gpointer d
 	/* We need to use the selected node here, as for search folders
 	   if we'd rely on the parent node of the changed item we would
 	   enable the wrong menu options */
-	nodePtr	node = feedlist_get_selected ();
+	Node	*node = feedlist_get_selected ();
 
 	if (!node) {
 		liferea_shell_update_feed_menu (TRUE, FALSE, FALSE);
@@ -438,8 +436,8 @@ liferea_shell_update_node_actions (gpointer obj, gchar *unusedNodeId, gpointer d
 
 	gboolean allowModify = (NODE_SOURCE_TYPE (node->source->root)->capabilities & NODE_SOURCE_CAPABILITY_WRITABLE_FEEDLIST);
 	liferea_shell_update_feed_menu (allowModify, TRUE, allowModify);
-	liferea_shell_update_update_menu ((NODE_TYPE (node)->capabilities & NODE_CAPABILITY_UPDATE) ||
-	                                  (NODE_TYPE (node)->capabilities & NODE_CAPABILITY_UPDATE_CHILDS));
+	liferea_shell_update_update_menu ((NODE_PROVIDER (node)->capabilities & NODE_CAPABILITY_UPDATE) ||
+	                                  (NODE_PROVIDER (node)->capabilities & NODE_CAPABILITY_UPDATE_CHILDS));
 
 	// Needs to be last as liferea_shell_update_update_menu() default enables actions
 	if (IS_FEED (node))
@@ -610,12 +608,53 @@ on_close (GtkWidget *widget, GdkEvent *event, gpointer user_data)
 static gboolean
 on_window_resize_cb (gpointer user_data)
 {
+	gint 		vpane_pos = 0, hpane_pos = 0, wpane_pos = 0;
+	GtkWidget	*pane;
+	GdkWindow	*gdk_window;
+	GtkAllocation	allocation;
+
 	shell->resizeTimer = 0;
 
 	/* If we are in auto layout mode we ask the itemview to calculate it again */
 	if (NODE_VIEW_MODE_AUTO == itemview_get_layout ())
 		itemview_set_layout (NODE_VIEW_MODE_AUTO);
 
+	/* Sanity check pane sizes for 0 values after layout switch
+	   this is necessary when switching wide mode to maximized window */
+
+	/* get pane proportions */
+	pane = liferea_shell_lookup ("leftpane");
+	vpane_pos = gtk_paned_get_position (GTK_PANED (pane));
+
+	pane = liferea_shell_lookup ("normalViewPane");
+	hpane_pos = gtk_paned_get_position (GTK_PANED (pane));
+
+	pane = liferea_shell_lookup ("wideViewPane");
+	wpane_pos = gtk_paned_get_position (GTK_PANED (pane));
+
+	// The following code partially duplicates liferea_shell_restore_layout()
+
+	/* a) set leftpane to 1/3rd of window size if too large */
+	gdk_window = gtk_widget_get_window (GTK_WIDGET (shell->window));
+	if (gdk_window_get_width (gdk_window) * 95 / 100 <= vpane_pos || 0 == vpane_pos) {
+		debug(DEBUG_GUI, "Fixing leftpane position");
+		gtk_paned_set_position (GTK_PANED (liferea_shell_lookup ("leftpane")), gdk_window_get_width (gdk_window) / 3);
+	}
+
+	/* b) set normalViewPane to 50% container height if too large */
+	gtk_widget_get_allocation (GTK_WIDGET (liferea_shell_lookup ("normalViewPane")), &allocation);
+	if ((allocation.height * 95 / 100 <= hpane_pos) || 0 == hpane_pos) {
+		debug(DEBUG_GUI, "Fixing normalViewPane position");
+		gtk_paned_set_position (GTK_PANED (liferea_shell_lookup ("normalViewPane")), allocation.height / 2);
+	}
+
+	/* c) set wideViewPane to 50% container width if too large */
+	gtk_widget_get_allocation (GTK_WIDGET (liferea_shell_lookup ("wideViewPane")), &allocation);
+	if ((allocation.width * 95 / 100 <= wpane_pos) || 0 == wpane_pos) {
+		debug(DEBUG_GUI, "Fixing wideViewPane position");
+		gtk_paned_set_position (GTK_PANED (liferea_shell_lookup ("wideViewPane")), allocation.width / 2);
+	}
+
 	return FALSE;
 }
 
@@ -634,6 +673,9 @@ on_configure_event (GtkWidget *window, GdkEvent *event, gpointer user_data)
 static gboolean
 on_window_state_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
 {
+	if(!shell)
+		return FALSE;
+
 	if (event->type == GDK_WINDOW_STATE) {
 		GdkWindowState changed = ((GdkEventWindowState*)event)->changed_mask, state = ((GdkEventWindowState*)event)->new_window_state;
 
@@ -770,6 +812,12 @@ on_prefbtn_clicked (GSimpleAction *action, GVariant *parameter, gpointer user_da
 	preferences_dialog_open ();
 }
 
+static void
+on_manage_plugins_clicked (GSimpleAction *action, GVariant *parameter, gpointer user_data)
+{
+	liferea_plugins_manage_dialog (GTK_WINDOW (liferea_shell_get_window ()));
+}
+
 static void
 on_searchbtn_clicked (GSimpleAction *action, GVariant *parameter, gpointer user_data)
 {
@@ -787,36 +835,39 @@ on_about_activate (GSimpleAction *action, GVariant *parameter, gpointer user_dat
 	gtk_widget_show (dialog);
 }
 
-static void
-liferea_shell_add_html_tab (const gchar *file, const gchar *name)
+static gchar *
+create_help_url (const gchar *file)
 {
 	gchar *filepattern = g_strdup_printf (PACKAGE_DATA_DIR "/" PACKAGE "/doc/html/%s", file);
 	gchar *filename = common_get_localized_filename (filepattern);
 	gchar *fileuri = g_strdup_printf ("file://%s", filename);
 
-	browser_tabs_add_new (fileuri, name, TRUE);
-
 	g_free (filepattern);
 	g_free (filename);
-	g_free (fileuri);
+	return fileuri;
+}
+
+static void
+on_action_discover_feeds (GSimpleAction *action, GVariant *parameter, gpointer user_data)
+{
+        LifereaBrowser *browser = browser_tabs_add_new ("https://lwindolf.github.io/rss-finder/?target=_self&show-title=false", _("Discover Feeds"), TRUE);
+	g_object_set (G_OBJECT (browser), "hidden-urlbar", TRUE, NULL);
 }
 
 static void
 on_topics_activate (GSimpleAction *action, GVariant *parameter, gpointer user_data)
 {
-	liferea_shell_add_html_tab ("topics_%s.html", _("Help Topics"));
+	g_autofree gchar *url = create_help_url ("topics_%s.html");
+	LifereaBrowser *browser = browser_tabs_add_new (url, _("Help Topics"), TRUE);
+	g_object_set (G_OBJECT (browser), "hidden-urlbar", TRUE, NULL);
 }
 
 static void
-on_quick_reference_activate (GSimpleAction *action, GVariant *parameter, gpointer user_data)
+on_shortcuts_activate (GSimpleAction *action, GVariant *parameter, gpointer user_data)
 {
-	liferea_shell_add_html_tab ("reference_%s.html", _("Quick Reference"));
-}
-
-static void
-on_faq_activate (GSimpleAction *action, GVariant *parameter, gpointer user_data)
-{
-	liferea_shell_add_html_tab ("faq_%s.html", _("FAQ"));
+	GtkWindow *window = GTK_WINDOW (liferea_dialog_new ("shortcuts"));
+	gtk_widget_show_all (GTK_WIDGET (window));
+	gtk_window_present (window);
 }
 
 static void
@@ -864,18 +915,37 @@ on_menu_export (GSimpleAction *action, GVariant *parameter, gpointer user_data)
 	export_OPML_file ();
 }
 
+static void
+on_print_activate (GSimpleAction *action, GVariant *parameter, gpointer user_data)
+{
+	LifereaBrowser *browser;
+	GtkWidget *webview;
+	
+	// First see if a browser tab is active
+	browser = browser_tabs_get_active_htmlview ();
+
+	// If not print the item view
+	if (!browser)
+		g_object_get (G_OBJECT (shell->itemview), "html-view", &browser, NULL);
+
+	g_return_if_fail (browser != NULL);
+
+	g_object_get (G_OBJECT (browser), "renderwidget", &webview, NULL);
+	liferea_web_view_print (LIFEREA_WEB_VIEW (webview));
+}
+
 /* methods to receive URLs which were dropped anywhere in the main window */
 static void
 liferea_shell_URL_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y, GtkSelectionData *data, guint info, guint time_received)
 {
-	gchar		*tmp1, *tmp2, *freeme;
+	gchar		*tmp1, *tmp2;
 	GtkWidget	*mainwindow;
 	GtkAllocation	alloc;
 	GtkTreeView	*treeview;
 	GtkTreeModel	*model;
 	GtkTreePath	*path;
 	GtkTreeIter	iter;
-	nodePtr		node;
+	Node		*node;
 	gint		tx, ty;
 
 	g_return_if_fail (gtk_selection_data_get_data (data) != NULL);
@@ -900,9 +970,15 @@ liferea_shell_URL_received (GtkWidget *widget, GdkDragContext *context, gint x,
 
 	if ((gtk_selection_data_get_length (data) >= 0) && (gtk_selection_data_get_format (data) == 8)) {
 		/* extra handling to accept multiple drops */
-		freeme = tmp1 = g_strdup ((gchar *) gtk_selection_data_get_data (data));
+		g_autofree gchar *freeme = tmp1 = g_strdup ((gchar *) gtk_selection_data_get_data (data));
 		while ((tmp2 = strsep (&tmp1, "\n\r"))) {
 			if (strlen (tmp2)) {
+				/* Check for valid URI */
+				if (!g_uri_parse_scheme (tmp2)) {
+					gtk_drag_finish (context, FALSE, FALSE, time_received);
+					return;
+				}
+
 				/* if the drop is over a node, select it so that feedlist_add_subscription()
 				 * adds it in the correct folder */
 				if (gtk_tree_view_get_dest_row_at_pos (treeview, tx, ty, &path, NULL)) {
@@ -913,11 +989,9 @@ liferea_shell_URL_received (GtkWidget *widget, GdkDragContext *context, gint x,
 					}
 					gtk_tree_path_free (path);
 				}
-				feedlist_add_subscription (g_strdup (tmp2), NULL, NULL,
-				                           FEED_REQ_PRIORITY_HIGH);
+				feedlist_add_subscription_by_url (tmp2);
 			}
 		}
-		g_free (freeme);
 		gtk_drag_finish (context, TRUE, FALSE, time_received);
 	} else {
 		gtk_drag_finish (context, FALSE, FALSE, time_received);
@@ -947,13 +1021,6 @@ liferea_shell_setup_URL_receiver (void)
 	                  G_CALLBACK (liferea_shell_URL_received), NULL);
 }
 
-static void
-on_action_open_enclosure (GSimpleAction *action, GVariant *parameter, gpointer user_data)
-{
-	LifereaShell *shell = LIFEREA_SHELL (user_data);
-	itemview_open_next_enclosure (shell->itemview);
-}
-
 static const GActionEntry liferea_shell_gaction_entries[] = {
 	{"update-all", on_menu_update_all, NULL, NULL, NULL},
 	{"mark-all-feeds-read", on_action_mark_all_read, NULL, NULL, NULL},
@@ -967,13 +1034,15 @@ static const GActionEntry liferea_shell_gaction_entries[] = {
 	{"zoom-in", on_action_zoomin_activate, NULL, NULL, NULL},
 	{"zoom-out", on_action_zoomout_activate, NULL, NULL, NULL},
 	{"zoom-reset", on_action_zoomreset_activate, NULL, NULL, NULL},
+	{"discover-feeds", on_action_discover_feeds, NULL, NULL, NULL},
 	{"show-update-monitor", on_menu_show_update_monitor, NULL, NULL, NULL},
 	{"show-preferences", on_prefbtn_clicked, NULL, NULL, NULL},
+	{"manage-plugins", on_manage_plugins_clicked, NULL, NULL, NULL},
 	{"search-feeds", on_searchbtn_clicked, NULL, NULL, NULL},
 	{"show-help-contents", on_topics_activate, NULL, NULL, NULL},
-	{"show-help-quick-reference", on_quick_reference_activate, NULL, NULL, NULL},
-	{"show-help-faq", on_faq_activate, NULL, NULL, NULL},
+	{"show-shortcuts", on_shortcuts_activate, NULL, NULL, NULL},
 	{"show-about", on_about_activate, NULL, NULL, NULL},
+	{"print", on_print_activate, NULL, NULL, NULL},
 
 	/* Parameter type must be NULL for toggle. */
 	{"fullscreen", NULL, NULL, "@b false", on_menu_fullscreen_activate},
@@ -984,8 +1053,7 @@ static const GActionEntry liferea_shell_gaction_entries[] = {
 	{"remove-item", on_action_remove_item, "t", NULL, NULL},
 	{"launch-item-in-tab", on_action_launch_item_in_tab, "t", NULL, NULL},
 	{"launch-item-in-browser", on_action_launch_item_in_browser, "t", NULL, NULL},
-	{"launch-item-in-external-browser", on_action_launch_item_in_external_browser, "t", NULL, NULL},
-	{"open-item-enclosure", on_action_open_enclosure, "t", NULL, NULL},
+	{"launch-item-in-external-browser", on_action_launch_item_in_external_browser, "t", NULL, NULL}
 };
 
 static const GActionEntry liferea_shell_add_gaction_entries[] = {
@@ -997,6 +1065,7 @@ static const GActionEntry liferea_shell_add_gaction_entries[] = {
 };
 
 static const GActionEntry liferea_shell_feed_gaction_entries[] = {
+	{"mark-feed-as-read", on_action_mark_all_read, "s", NULL, NULL},
 	{"mark-selected-feed-as-read", on_action_mark_all_read, NULL, NULL, NULL},
 	{"update-selected", on_menu_update, NULL, NULL, NULL}
 };
@@ -1012,8 +1081,7 @@ static const GActionEntry liferea_shell_item_gaction_entries[] = {
 	{"remove-selected-item", on_action_remove_item, NULL, NULL, NULL},
 	{"launch-selected-item-in-tab", on_action_launch_item_in_tab, NULL, NULL, NULL},
 	{"launch-selected-item-in-browser", on_action_launch_item_in_browser, NULL, NULL, NULL},
-	{"launch-selected-item-in-external-browser", on_action_launch_item_in_external_browser, NULL, NULL, NULL},
-	{"open-selected-item-enclosure", on_action_open_enclosure, NULL, NULL, NULL}
+	{"launch-selected-item-in-external-browser", on_action_launch_item_in_external_browser, NULL, NULL, NULL}
 };
 
 static void
@@ -1090,7 +1158,7 @@ email_the_author(GSimpleAction *action, GVariant *parameter, gpointer user_data)
 		g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error);
 
 		if (error && (0 != error->code)) {
-			debug2 (DEBUG_GUI, "Email command failed: %s : %s", argv[0], error->message);
+			debug (DEBUG_GUI, "Email command failed: %s : %s", argv[0], error->message);
 			liferea_shell_set_important_status_bar (_("Email command failed: %s"), error->message);
 			g_error_free (error);
 		} else {
@@ -1115,15 +1183,13 @@ static const GActionEntry liferea_shell_link_gaction_entries[] = {
 	{"email-the-author", email_the_author, "t", NULL, NULL}
 };
 
-static gboolean
-liferea_shell_restore_layout (gpointer user_data)
+static void
+liferea_shell_restore_layout (void)
 {
 	GdkWindow	*gdk_window;
 	GtkAllocation	allocation;
 	gint		last_vpane_pos, last_hpane_pos, last_wpane_pos;
 
-	liferea_shell_restore_position ();
-
 	/* This only works after the window has been restored, so we do it last. */
 	conf_get_int_value (LAST_VPANE_POS, &last_vpane_pos);
 	conf_get_int_value (LAST_HPANE_POS, &last_hpane_pos);
@@ -1145,13 +1211,11 @@ liferea_shell_restore_layout (gpointer user_data)
 	if ((allocation.width * 95 / 100 <= last_wpane_pos) || 0 == last_wpane_pos)
 		last_wpane_pos = allocation.width / 2;
 
-	debug3 (DEBUG_GUI, "Restoring pane proportions (left:%d normal:%d wide:%d)", last_vpane_pos, last_hpane_pos, last_wpane_pos);
+	debug (DEBUG_GUI, "Restoring pane proportions (left:%d normal:%d wide:%d)", last_vpane_pos, last_hpane_pos, last_wpane_pos);
 
 	gtk_paned_set_position (GTK_PANED (liferea_shell_lookup ("leftpane")), last_vpane_pos);
 	gtk_paned_set_position (GTK_PANED (liferea_shell_lookup ("normalViewPane")), last_hpane_pos);
 	gtk_paned_set_position (GTK_PANED (liferea_shell_lookup ("wideViewPane")), last_wpane_pos);
-	
-	return FALSE;
 }
 
 static void
@@ -1161,13 +1225,13 @@ liferea_shell_restore_state (const gchar *overrideWindowState)
 	gchar		*toolbar_style;
 	gint		resultState;
 
-	debug0 (DEBUG_GUI, "Setting toolbar style");
+	debug (DEBUG_GUI, "Setting toolbar style");
 
 	toolbar_style = conf_get_toolbar_style ();
 	liferea_shell_set_toolbar_style (toolbar_style);
 	g_free (toolbar_style);
 
-	debug0 (DEBUG_GUI, "Restoring window position");
+	debug (DEBUG_GUI, "Restoring window position");
 	/* Realize needed so that the window structure can be
 	   accessed... otherwise we get a GTK warning when window is
 	   shown by clicking on notification icon or when theme
@@ -1183,7 +1247,7 @@ liferea_shell_restore_state (const gchar *overrideWindowState)
 	/* Initialize with last saved state */
 	conf_get_int_value (LAST_WINDOW_STATE, &resultState);
 
-	debug2 (DEBUG_GUI, "Previous window state indicators: dconf=%d, CLI switch=%s", resultState, overrideWindowState);
+	debug (DEBUG_GUI, "Previous window state indicators: dconf=%d, CLI switch=%s", resultState, overrideWindowState);
 
 	/* Override with command line options */
 	if (!g_strcmp0 (overrideWindowState, "hidden"))
@@ -1194,13 +1258,13 @@ liferea_shell_restore_state (const gchar *overrideWindowState)
 	/* And set the window to the resulting state */
 	switch (resultState) {
 		case MAINWINDOW_HIDDEN:
-			debug0 (DEBUG_GUI, "Restoring window state 'hidden (to tray)'");
+			debug (DEBUG_GUI, "Restoring window state 'hidden (to tray)'");
 			gtk_widget_hide (GTK_WIDGET (shell->window));
 			break;
 		case MAINWINDOW_SHOWN:
 		default:
 			/* Safe default is always to show window */
-			debug0 (DEBUG_GUI, "Restoring window state 'shown'");
+			debug (DEBUG_GUI, "Restoring window state 'shown'");
 			gtk_widget_show (GTK_WIDGET (shell->window));
 	}
 	
@@ -1211,11 +1275,6 @@ liferea_shell_restore_state (const gchar *overrideWindowState)
 		gtk_container_child_set (GTK_CONTAINER (liferea_shell_lookup ("wideViewPane")), liferea_shell_lookup ("wideViewItems"),
 			"resize", TRUE, NULL);
 	}
-	
-	/* Need to run asynchronous otherwise pane widget allocation is reported
-	   wrong, maybe it is running to early as we realize only above. Also
-	   only like this window position restore works properly. */
-	g_idle_add (liferea_shell_restore_layout, NULL);
 }
 
 static const gchar * liferea_accels_update_all[] = {"<Control>u", NULL};
@@ -1231,7 +1290,7 @@ static const gchar * liferea_accels_zoom_out[] = {"<Control>minus", NULL};
 static const gchar * liferea_accels_zoom_reset[] = {"<Control>0", NULL};
 static const gchar * liferea_accels_search_feeds[] = {"<Control>f", NULL};
 static const gchar * liferea_accels_show_help_contents[] = {"F1", NULL};
-static const gchar * liferea_accels_open_selected_item_enclosure[] = {"<Control>o", NULL};
+static const gchar * liferea_accels_show_shortcuts[] = {"<Control>question", NULL};
 static const gchar * liferea_accels_launch_item_in_external_browser[] = {"<Control>d", NULL};
 
 void
@@ -1243,14 +1302,20 @@ liferea_shell_create (GtkApplication *app, const gchar *overrideWindowState, gin
 	gint			mode;
 	FeedListView	*feedListView;
 
-	debug_enter ("liferea_shell_create");
 
 	g_object_new (LIFEREA_SHELL_TYPE, NULL);
+	g_assert(shell);
 
 	shell->window = GTK_WINDOW (liferea_shell_lookup ("mainwindow"));
 
 	gtk_window_set_application (GTK_WINDOW (shell->window), app);
 
+	conf_bind (LAST_WINDOW_WIDTH, shell->window, "default-width", G_SETTINGS_BIND_DEFAULT);
+	conf_bind (LAST_WINDOW_HEIGHT, shell->window, "default-height", G_SETTINGS_BIND_DEFAULT);
+	conf_bind (LAST_VPANE_POS, liferea_shell_lookup ("leftpane"), "position", G_SETTINGS_BIND_DEFAULT);
+	conf_bind (LAST_HPANE_POS, liferea_shell_lookup ("normalViewPane"), "position", G_SETTINGS_BIND_DEFAULT);
+	conf_bind (LAST_WPANE_POS, liferea_shell_lookup ("wideViewPane"), "position", G_SETTINGS_BIND_DEFAULT);
+
 	/* Add GActions to application */
 	shell->generalActions = G_ACTION_GROUP (g_simple_action_group_new ());
 	g_action_map_add_action_entries (G_ACTION_MAP(shell->generalActions), liferea_shell_gaction_entries, G_N_ELEMENTS (liferea_shell_gaction_entries), NULL);
@@ -1274,9 +1339,12 @@ liferea_shell_create (GtkApplication *app, const gchar *overrideWindowState, gin
 
 	g_action_map_add_action_entries (G_ACTION_MAP(app), liferea_shell_link_gaction_entries, G_N_ELEMENTS (liferea_shell_link_gaction_entries), NULL);
 
+	/* 0.) setup plugin engine including mandatory base plugins that (for example the feed list or auth) might depend on */
+	shell->plugins = liferea_plugins_engine_get ();
+
 	/* 1.) menu creation */
 
-	debug0 (DEBUG_GUI, "Setting up menus");
+	debug (DEBUG_GUI, "Setting up menus");
 
 	shell->itemlist = itemlist_create ();
 
@@ -1302,8 +1370,8 @@ liferea_shell_create (GtkApplication *app, const gchar *overrideWindowState, gin
 	gtk_application_set_accels_for_action (app, "app.zoom-out", liferea_accels_zoom_out);
 	gtk_application_set_accels_for_action (app, "app.zoom-reset", liferea_accels_zoom_reset);
 	gtk_application_set_accels_for_action (app, "app.search-feeds", liferea_accels_search_feeds);
+	gtk_application_set_accels_for_action (app, "app.show-shortcuts", liferea_accels_show_shortcuts);
 	gtk_application_set_accels_for_action (app, "app.show-help-contents", liferea_accels_show_help_contents);
-	gtk_application_set_accels_for_action (app, "app.open-selected-item-enclosure", liferea_accels_open_selected_item_enclosure);
 	gtk_application_set_accels_for_action (app, "app.launch-item-in-external-browser", liferea_accels_launch_item_in_external_browser);
 
 	/* Toolbar */
@@ -1313,17 +1381,16 @@ liferea_shell_create (GtkApplication *app, const gchar *overrideWindowState, gin
 
 	/* 2.) setup containers */
 
-	debug0 (DEBUG_GUI, "Setting up widget containers");
+	debug (DEBUG_GUI, "Setting up widget containers");
 
 	gtk_grid_attach_next_to (GTK_GRID (liferea_shell_lookup ("vbox1")), shell->toolbar, NULL, GTK_POS_TOP, 1,1);
 
 	gtk_widget_show_all(GTK_WIDGET(shell->toolbar));
-	render_init_theme_colors (GTK_WIDGET (shell->window));
 	g_signal_connect (G_OBJECT (shell->window), "style-updated", G_CALLBACK(liferea_shell_rebuild_css), NULL);
 
 	/* 3.) setup status bar */
 
-	debug0 (DEBUG_GUI, "Setting up status bar");
+	debug (DEBUG_GUI, "Setting up status bar");
 
 	shell->statusbar = GTK_STATUSBAR (liferea_shell_lookup ("statusbar"));
 	shell->statusbarLocked = FALSE;
@@ -1337,18 +1404,18 @@ liferea_shell_create (GtkApplication *app, const gchar *overrideWindowState, gin
 
 	/* 4.) setup tabs */
 
-	debug0 (DEBUG_GUI, "Setting up tabbed browsing");
+	debug (DEBUG_GUI, "Setting up tabbed browsing");
 	shell->tabs = browser_tabs_create (GTK_NOTEBOOK (liferea_shell_lookup ("browsertabs")));
 
 	/* 5.) setup feed list */
 
-	debug0 (DEBUG_GUI, "Setting up feed list");
+	debug (DEBUG_GUI, "Setting up feed list");
 	shell->feedlistViewWidget = GTK_TREE_VIEW (liferea_shell_lookup ("feedlist"));
 	feedListView = feed_list_view_create (shell->feedlistViewWidget);
 
 	/* 6.) setup menu sensivity */
 
-	debug0 (DEBUG_GUI, "Initialising menus");
+	debug (DEBUG_GUI, "Initialising menus");
 
 	/* On start, no item or feed is selected, so menus should be insensitive */
 	liferea_shell_update_item_menu (FALSE);
@@ -1359,13 +1426,13 @@ liferea_shell_create (GtkApplication *app, const gchar *overrideWindowState, gin
 
 	/* 7.) setup item view */
 
-	debug0 (DEBUG_GUI, "Setting up item view");
+	debug (DEBUG_GUI, "Setting up item view");
 
 	shell->itemview = itemview_create (GTK_WIDGET (shell->window));
 
         /* 8.) load icons as required */
 
-        debug0 (DEBUG_GUI, "Loading icons");
+        debug (DEBUG_GUI, "Loading icons");
 
         icons_load ();
 
@@ -1390,6 +1457,9 @@ liferea_shell_create (GtkApplication *app, const gchar *overrideWindowState, gin
 		g_free (id);
 	}
 
+	/* Load shell state and bind for persisting */
+	liferea_shell_restore_layout ();
+
 	/* 12. Setup shell window signals, only after all widgets are ready */
 	g_signal_connect (shell->feedlist, "new-items",
 	                  G_CALLBACK (liferea_shell_update_unread_stats), shell->feedlist);
@@ -1414,26 +1484,22 @@ liferea_shell_create (GtkApplication *app, const gchar *overrideWindowState, gin
 	g_signal_connect (G_OBJECT (shell->window), "configure_event", G_CALLBACK(on_configure_event), shell);
 	g_signal_connect (G_OBJECT (shell->window), "key_press_event", G_CALLBACK(on_key_press_event), shell);
 
-	/* 13. Setup shell plugins */
-	if(0 == pluginsDisabled) {
-		shell->extensions = peas_extension_set_new (PEAS_ENGINE (liferea_plugins_engine_get_default ()),
-				                     LIFEREA_TYPE_SHELL_ACTIVATABLE, "shell", shell, NULL);
-
-		liferea_plugins_engine_set_default_signals (shell->extensions, shell);
-	}
+	/* 13. Setup plugins */
+	if(!pluginsDisabled)
+		liferea_plugins_engine_register_shell_plugins (shell);
 
 	/* 14. Rebuild search folders if needed */
 	if (searchFolderRebuild)
 		vfolder_foreach (vfolder_rebuild);
 
-	debug_exit ("liferea_shell_create");
 }
 
 void
 liferea_shell_destroy (void)
 {
+	g_object_unref (shell->plugins);
+
 	liferea_shell_save_position ();
-	g_object_unref (shell->extensions);
 	g_object_unref (shell->tabs);
 	g_object_unref (shell->feedlist);
 	g_object_unref (shell->itemview);
@@ -1501,7 +1567,6 @@ liferea_shell_get_window (void)
 void
 liferea_shell_rebuild_css (void)
 {
-	render_init_theme_colors (GTK_WIDGET (shell->window));
 	itemview_style_update ();
 }
 
